==> hakk_builder-0.2.1.tar.gz: tool/test.dart <==
import 'package:hakk_builder/basic.dart';

main(){
//  _run(identity());
//  _run(helloWorld());
//  _run(test());
//  _run(subroutineTest());
//  _run(logicTest());
//  _run(ifTest());
//  _run(whileTest());
//  _run(incTest());
//  _run(nestedWhileTest());
//  _run(mathExpressionTest());
//  _run(popTest());
//  _run(powerTest());
//  _run(memTest());
//  _run(inlineTest());
//  _run(mobilityTest());
//  _run(callTest());
//  _run(forTest());
//  _run(facingTest());
//  _run(vectorToDirectionTest());
  _run(moveTowardEnemy());
  
//  _run(MMUTesting());
}

_run(String hakk){
  print('');
  print(hakk);
  print('***************************************');
  new Debugger().dump();
}

String MMUTesting(){
  hakk('test');
  Block.system(0, 1);

  FOR(core0 << 0, core0 < 10, () => inc(core0));
    output(core0);
  NEXT();
  
  return build();
}

String moveTowardEnemy(){
  hakk('finder');
  Block.system(0, 1);
  
  core0 << true;

  WHILE(core0);
    core0 << faceEnemy();
    IF(core0);
      moveForward();
      moveForward();
    ENDIF();
  ENDWHILE();
  core0 << faceEnemy();
  
  return build();
}

String vectorToDirectionTest(){
  hakk('example');
  Block.system(0, 1);

  CALL('dovec', [0, -1]);
  CALL('dovec', [0, 1]);
  CALL('dovec', [-1, 0]);
  CALL('dovec', [1, 0]);
  CALL('dovec', [1, 1]);
  
  SUB('dovec', [core2, core3]);
    core4 << vectorToDirection(core2, core3);
    IF (core4 == NORTH);
      output('north');
    ENDIF();
    IF (core4 == SOUTH);
      output('south');
    ENDIF();
    IF (core4 == EAST);
      output('east');
    ENDIF();
    IF (core4 == WEST);
      output('west');
    ENDIF();
    IF (core4 == -1);
      output('invalid vector!');
    ENDIF();
  RETURN();
 
  return build();
}

String facingTest(){
  hakk('facing_test');
  Block.system(0, 1);
  
  core0 << facing();
  output(faceDirection(SOUTH));
   
  return build();
}

String forTest(){
  hakk('for_example');
  Block.system(0, 1);
  
  FOR(core0 << 0, core0 < 10, () => inc(core0));
    output('in core 0');
    FOR(core1 << 0, core1 < 5, () => inc(core1));
      output('   in core 1');
    NEXT(); 
  NEXT();
   
  return build();
}

String callTest(){
  hakk('call_test');
  Block.system(0, 1);
  
  CALL('test');
  
  output('finished.');
  
  SUB('test');
    output('yes!');
  RETURN();
  
  return build();
}

String mobilityTest(){
  hakk('mobility_test');
  Block.system(0, 1);
  
  output(facing());
  IF(moveForward() == false);
    moveBack();
    moveBack();
    moveBack();
    core0 << random(2);
    IF(core0 == 0);
      turnRight();
    ENDIF();
    IF(core0 == 1);
      turnLeft();
    ENDIF();
  ENDIF(); 
  GOTO('core');
 
  return build();
}

String inlineTest(){
  hakk('inline_example');
  Block.system(0, 1);
  
  core0 << 42;
  output(core0);
  inline('set: [core.0, 24]');
  output(core0);
  
  core0 << 42;
  output(core0);
  inline([
          'set: [core.0, 24]',
          'add: [core.1, core.0, 24]',
          'out: core.1'
          ]);
  
  return build();
}

String memTest(){
  hakk('mem_example');
  Block.system(0, 1);
  Block.memory('m1', 0, -1);
 
  core0 << 5;
  mem('m1', 0) << core0;
  
  final mem0 = mem('m1', 0);
  final mem1 = mem('m1', 1);
  
  mem1 << mem0 * 3;
  
  output(mem1);
  
  return build();
}

String powerTest(){
  hakk('power_test');
  Block.system(0, 1);
  
  core0 << 2;
  dec(core0);
  
  output(power(val(3) * val(4), core0 + core0));
  
  return build();
}

String popTest(){
     hakk('stack_example');
     Block.system(0, 1);
     
     // push 3 values onto the stack, the last value being the "amount" of
     // other values pushed.
     core0 << 42;
     // core0 can safely be used again because it's value is now pushed onto the
     // stack.
     push(core0);
     push(-42);
     push(2);
     
     // ..somewhere else in code
     
     //pop the first value, which contains the amount of other values to pop
     core0 << pop();
     output('how many items to retrieve:');
     output(core0);
     core1 << 0;
    
     output('');
     output('Items:');
     WHILE(core1 <= core0);
       output(pop());
       inc(core1);
     ENDWHILE();
     
     // outputs:
     // -42
     // 42
  
  return build();
}

String mathExpressionTest(){
  hakk('math_expression_test');
  Block.system(0, 1);
  
  core0 << (core0 + val(1)) / (core0 * val(2));
  
  output(core0);
  
  output(random(5) < (random(5) + random(5)));
  
  return build();
}

String incTest(){
  hakk('inc_test');
  Block.system(0, 1);
  
  core0 << 0;
  inc(core0);
  output(core0);
  return build();
}

String nestedWhileTest(){
  hakk('nested_while_test');
  Block.system(0, 1);
      
  core1 << 0;
  core0 << 0;
  WHILE(core0 < 10);
    output('hello!');
    WHILE((core1 < val(10)) & (core0 >= val(5)));
      output('       world!');
      inc(core1);
    ENDWHILE();
    inc(core0);
  ENDWHILE();
  output('finished');
  return build();
}

String whileTest(){
  hakk('while_test');
  Block.system(0, 1);
      
  WHILE(true);
    inc(core0);
    output(core0);
    
    IF (core0 >= 10); 
      GOTO('finished');
    ENDIF();

  ENDWHILE();
  label('finished');
  output('finished');
  return build();
}

String ifTest(){
  hakk('if_test');
  Block.system(0, 1);
  Block.instructions('foo', 0, -1);
  
  codeBlock('foo');
    IF(core10);
      output('the end');
    ENDIF();
  
  codeBlock('core');
    IF(val(5) < 10);
      core10 << true;
      IF(true);
        output('we have found a nested truth before all else');
      ENDIF();
      output("we have found the truth");
    ENDIF();
    
    output('and then continued onward');
    GOTO('foo');
   
  return build();
}

String logicTest(){
  hakk('logic_test');
  Block.system(0, 1);
  
  core0 << 43;
  output((val(15) < core0) & (val(42) == core0));
  core0 << val(42);
  core0 << ((val(15) < core0) & (val(42) == core0));
  output(core0);
  return build();
}

String subroutineTest(){
  hakk('subroutine_test');
  
  Block.system(0, 1);
  Block.memory('m1', 0, -1);
  
  SUB('doubler', [core4]);
    RETURN(core4 * 2);
  
  SUB('adder', [core2, core3]);     
    RETURN(GOSUB('doubler', [core2]) + GOSUB('doubler', [core3]));
         
  output(GOSUB('adder', [5, 5]));
   
  output("Finished!");
  EXIT(0);
   
  return build();
}

String test(){
  hakk('test');
  
  Block.system(0, 1);
  Block.instructions('foo', 0, -1);
  
  // these next two lines are equivalent
  set(core0, 10);
  core0 << 10;
  
  core0 << val(2) - val(2) + val(3) * core3;
   
  core2 << (core0 + val(10)) + cosine(val(5) + val(5));
  
  core1 << cosine(val(5) + sine(core3));
  
  core0 << random(core1) + val(3);  
  
  wait(val(5) + core0);
  push(5);
  core10 << pop();
  output(core10);

  codeBlock('foo');
  
  return build();
}

String helloWorld(){
  hakk('hello_world');
  Block.system(0, 1);
  
  output("Hello World!");

  return build();
}

String identity(){
  hakk('identity');
  Block.system(0, 1);
  
  label('top');
  GOTO('top');
    
  return build();
}

==> hakk_builder-0.2.2.tar.gz: tool/test.dart <==
import 'package:hakk_builder/basic.dart';

main(){
//  _run(identity());
//  _run(helloWorld());
//  _run(test());
//  _run(subroutineTest());
//  _run(logicTest());
//  _run(ifTest());
//  _run(whileTest());
//  _run(incTest());
//  _run(nestedWhileTest());
//  _run(mathExpressionTest());
//  _run(popTest());
//  _run(powerTest());
//  _run(memTest());
//  _run(inlineTest());
//  _run(mobilityTest());
//  _run(callTest());
//  _run(forTest());
//  _run(facingTest());
//  _run(vectorToDirectionTest());
//  _run(moveTowardEnemy());
  _run(MMUTesting());
}

_run(String hakk){
  print('');
  print(hakk);
  print('***************************************');
  new Debugger().dump(verbose: true);
}


String MMUTesting(){
  hakk('test');
  Block.system(0, 1);
    
  SUB('do_eet', [core14, core15]);
    core13 << core14 * core15;
  RETURN(core13 * 5);
    
  FOR(core0 << 0, core0 < 10, () => inc(core0));
    core1 << GOSUB('do_eet', [core0, 3]);
    output(core1);
  NEXT();
 
  return build();
}

String moveTowardEnemy(){
  hakk('finder');
  Block.system(0, 1);
  
  core0 << true;

  WHILE(core0);
    core0 << faceEnemy();
    IF(core0);
      moveForward();
      moveForward();
    ENDIF();
  ENDWHILE();
  core0 << faceEnemy();
  
  return build();
}

String vectorToDirectionTest(){
  hakk('example');
  Block.system(0, 1);

  CALL('dovec', [0, -1]);
  CALL('dovec', [0, 1]);
  CALL('dovec', [-1, 0]);
  CALL('dovec', [1, 0]);
  CALL('dovec', [1, 1]);
  
  SUB('dovec', [core2, core3]);
    core4 << vectorToDirection(core2, core3);
    IF (core4 == NORTH);
      output('north');
    ENDIF();
    IF (core4 == SOUTH);
      output('south');
    ENDIF();
    IF (core4 == EAST);
      output('east');
    ENDIF();
    IF (core4 == WEST);
      output('west');
    ENDIF();
    IF (core4 == -1);
      output('invalid vector!');
    ENDIF();
  RETURN();
 
  return build();
}

String facingTest(){
  hakk('facing_test');
  Block.system(0, 1);
  
  core0 << facing();
  output(faceDirection(SOUTH));
   
  return build();
}

String forTest(){
  hakk('for_example');
  Block.system(0, 1);
  
  FOR(core0 << 0, core0 < 10, () => inc(core0));
    output('in core 0');
    FOR(core1 << 0, core1 < 5, () => inc(core1));
      output('   in core 1');
    NEXT(); 
  NEXT();
   
  return build();
}

String callTest(){
  hakk('call_test');
  Block.system(0, 1);
  
  CALL('test');
  
  output('finished.');
  
  SUB('test');
    output('yes!');
  RETURN();
  
  return build();
}

String mobilityTest(){
  hakk('mobility_test');
  Block.system(0, 1);
  
  output(facing());
  IF(moveForward() == false);
    moveBack();
    moveBack();
    moveBack();
    core0 << random(2);
    IF(core0 == 0);
      turnRight();
    ENDIF();
    IF(core0 == 1);
      turnLeft();
    ENDIF();
  ENDIF(); 
  GOTO('core');
 
  return build();
}

String inlineTest(){
  hakk('inline_example');
  Block.system(0, 1);
  
  core0 << 42;
  output(core0);
  inline('set: [core.0, 24]');
  output(core0);
  
  core0 << 42;
  output(core0);
  inline([
          'set: [core.0, 24]',
          'add: [core.1, core.0, 24]',
          'out: core.1'
          ]);
  
  return build();
}

String memTest(){
  hakk('mem_example');
  Block.system(0, 1);
  Block.memory('m1', 0, -1);
 
  core0 << 5;
  mem('m1', 0) << core0;
  
  final mem0 = mem('m1', 0);
  final mem1 = mem('m1', 1);
  
  mem1 << mem0 * 3;
  
  output(mem1);
  
  return build();
}

String powerTest(){
  hakk('power_test');
  Block.system(0, 1);
  
  core0 << 2;
  dec(core0);
  
  output(power(val(3) * val(4), core0 + core0));
  
  return build();
}

String popTest(){
     hakk('stack_example');
     Block.system(0, 1);
     
     // push 3 values onto the stack, the last value being the "amount" of
     // other values pushed.
     core0 << 42;
     // core0 can safely be used again because it's value is now pushed onto the
     // stack.
     push(core0);
     push(-42);
     push(2);
     
     // ..somewhere else in code
     
     //pop the first value, which contains the amount of other values to pop
     core0 << pop();
     output('how many items to retrieve:');
     output(core0);
     core1 << 0;
    
     output('');
     output('Items:');
     WHILE(core1 <= core0);
       output(pop());
       inc(core1);
     ENDWHILE();
     
     // outputs:
     // -42
     // 42
  
  return build();
}

String mathExpressionTest(){
  hakk('math_expression_test');
  Block.system(0, 1);
  
  core0 << (core0 + val(1)) / (core0 * val(2));
  
  output(core0);
  
  output(random(5) < (random(5) + random(5)));
  
  return build();
}

String incTest(){
  hakk('inc_test');
  Block.system(0, 1);
  
  core0 << 0;
  inc(core0);
  output(core0);
  return build();
}

String nestedWhileTest(){
  hakk('nested_while_test');
  Block.system(0, 1);
      
  core1 << 0;
  core0 << 0;
  WHILE(core0 < 10);
    output('hello!');
    WHILE((core1 < val(10)) & (core0 >= val(5)));
      output('       world!');
      inc(core1);
    ENDWHILE();
    inc(core0);
  ENDWHILE();
  output('finished');
  return build();
}

String whileTest(){
  hakk('while_test');
  Block.system(0, 1);
      
  WHILE(true);
    inc(core0);
    output(core0);
    
    IF (core0 >= 10); 
      GOTO('finished');
    ENDIF();

  ENDWHILE();
  label('finished');
  output('finished');
  return build();
}

String ifTest(){
  hakk('if_test');
  Block.system(0, 1);
  Block.instructions('foo', 0, -1);
  
  codeBlock('foo');
    IF(core10);
      output('the end');
    ENDIF();
  
  codeBlock('core');
    IF(val(5) < 10);
      core10 << true;
      IF(true);
        output('we have found a nested truth before all else');
      ENDIF();
      output("we have found the truth");
    ENDIF();
    
    output('and then continued onward');
    GOTO('foo');
   
  return build();
}

String logicTest(){
  hakk('logic_test');
  Block.system(0, 1);
  
  core0 << 43;
  output((val(15) < core0) & (val(42) == core0));
  core0 << val(42);
  core0 << ((val(15) < core0) & (val(42) == core0));
  output(core0);
  return build();
}

String subroutineTest(){
  hakk('subroutine_test');
  
  Block.system(0, 1);
  Block.memory('m1', 0, -1);
  
  SUB('doubler', [core4]);
    RETURN(core4 * 2);
  
  SUB('adder', [core2, core3]);     
    RETURN(GOSUB('doubler', [core2]) + GOSUB('doubler', [core3]));
         
  output(GOSUB('adder', [5, 5]));
   
  output("Finished!");
  EXIT(0);
   
  return build();
}

String test(){
  hakk('test');
  
  Block.system(0, 1);
  Block.instructions('foo', 0, -1);
  
  // these next two lines are equivalent
  set(core0, 10);
  core0 << 10;
  
  core0 << val(2) - val(2) + val(3) * core3;
   
  core2 << (core0 + val(10)) + cosine(val(5) + val(5));
  
  core1 << cosine(val(5) + sine(core3));
  
  core0 << random(core1) + val(3);  
  
  wait(val(5) + core0);
  push(5);
  core10 << pop();
  output(core10);

  codeBlock('foo');
  
  return build();
}

String helloWorld(){
  hakk('hello_world');
  Block.system(0, 1);
  
  output("Hello World!");

  return build();
}

String identity(){
  hakk('identity');
  Block.system(0, 1);
  
  label('top');
  GOTO('top');
    
  return build();
}
